---------------------------------------------------------------------
---------- Connecting all of the hooks from this file  --------------
---------------------------------------------------------------------

--[[

--Thinker that is executed when the player is in the climbing state
--Returns true if state function is executed
local function ClimbingThink(player, wall, wall_dist, wall_angle)

    --Is Jazz able to be climbing?
    local is_climbing = player.mo.state == S_CLIMBING
    if(is_climbing) then
        -- print(wall.flags)
        --Too far: Exiting the state
        if(player.jumpheld == 1) then
            player.mo.state = states[player.mo.state].nextstate
            print("exit!")
            
        --Close enough: Actual climbing
        else
            --Speed of climbing all four directions
            local CLIMBING_SPEED = 5*FRACUNIT

            --Ensure clinging to the wall
            P_SetObjectMomZ(player.mo, clamp(player.cmd.forwardmove, -1, 1)*CLIMBING_SPEED, false)
            P_InstaThrust(player.mo, wall_angle-ANGLE_90, clamp(player.cmd.sidemove, -1, 1)*CLIMBING_SPEED)
            P_Thrust(player.mo, wall_angle, FRACUNIT)

            -- player.lastlinehit = -1 --Making sure jazz won't be registering vertical lines he doesn't collide with
        end
    end
    return is_climbing
end
]]--
--Climb only when close enough to a wall and pressing jump
--Leaving when:
    --or Jumping
    --To far away


--Thinker for when the player is in the BulletStance state
local function BulletStanceThink(player)
    if(player.mo.state == S_BULLET_STANCE) then
        
        --Makes sure to display correct sprite depending on whether Jazz is airbone or not
        if(not P_IsObjectOnGround(player.mo)) then
            player.mo.sprite2 = SPR2_BUJU
        elseif(P_IsObjectOnGround(player.mo)) then
            player.mo.sprite2 = SPR2_BUSN
            
            --Skid on the ground
            if(player.speed > 0) then
                SpawnOnSide(player, MT_DUST)
            end
        end

        if(player.bullet_tics >= 0) then
            player.bullet_tics = $-1
        end
        
        if(player.bullet_tics % 5 == 0) then
        
            -- Bullet to be shot
            local bullet = P_SpawnMobjFromMobj(player.mo, 0, 0, 30*FRACUNIT, MT_BULL)
            bullet.target = player.mo
            bullet.angle = player.mo.angle
            
            --Bullet speed
            local bullet_side_speed = P_RandomRange(-3, 3)*FRACUNIT
            local bullet_vertical_speed = P_RandomRange(-3, 3)*FRACUNIT
            local bullet_forward_speed = mobjinfo[bullet.type].speed


            --Shooting the bullet bullet is under water 
            if(bullet.eflags & MFE_UNDERWATER ~= 0) then
                bullet_side_speed = $*2
                bullet_forward_speed = $/2
            end


            -- P_SetObjectMomZ(bullet, bullet_vertical_speed, false)
            P_InstaThrust(bullet, player.mo.angle + ANGLE_90, bullet_side_speed)
            P_Thrust(bullet, player.mo.angle, bullet_forward_speed)

            --Play souund
            S_StartSound(player.mo, sfx_bullet)
        end
    end
    
end

local function BotPlayerThink(player)

    if(player.mo.eflags & MFE_JUSTHITFLOOR and not S_SoundPlaying(player.mo, sfx_land)) then
        S_StartSound(player.mo, sfx_land)
        SpawnOnSide(player, MT_SPINDUST)
    end

    --Start shooting the Bullet projectile
    if(player.spinheld == 1 and player.pflags & PF_STASIS == 0) then
        player.mo.state = S_BULLET_STANCE
    end
    BulletStanceThink(player)

    
    --If not yet climbing, either start climbing or stop climbing
    if(player.climbing == 0) then
        
        if(player.lastlinehit ~= -1 and player.powers[pw_pushing] ~= 0 --If hitting a wall
        and player.tf_mode == TF_MODES.BOT_MODE --While in the robot mode
        and player.cmd.buttons & BT_JUMP ~= 0 --While holding jump button
        and player.jumpspecial >= 1 --If special is ready
        and (abs(player.cmd.forwardmove) > 20 or abs(player.cmd.sidemove) > 20)) then --While moving significantly
            
        
            --Information about the wall
            local climb_wall = lines[player.lastlinehit]
            -- print("L: "..player.lastlinehit)
            -- print("T: "..climb_wall.tag)
            -- print("S: "..climb_wall.special)
            --Extra checks for the wall
            if(climb_wall.flags & ML_NOCLIMB == 0 and climb_wall.special ~= 41) then
            
                --Wall x and y points
                local wall_x, wall_y = P_ClosestPointOnLine(player.mo.x, player.mo.y, climb_wall)
                --Angle between Jazz position and the closes point on the wall
                local wall_angle = R_PointToAngle2(player.mo.x, player.mo.y, wall_x, wall_y)
                --Distance between Jazz and closest point on the walll
                local wall_distance = R_PointToDist2(player.mo.x, player.mo.y, wall_x, wall_y)

                player.charability = CA_GLIDEANDCLIMB
                player.climbing = 5
                S_StartSound(player.mo, sfx_climb)
            end
        
        --Stop climbing by removing the ability
        elseif(player.charability == CA_GLIDEANDCLIMB and player.climbing == 0) then
            player.charability = skins[player.mo.skin].ability
        end
    end

    
end

local function BulletMobjThinker(bullet)
    --Underwater spread of the bullet
    if(bullet.eflags & MFE_UNDERWATER) then
        bullet.momx = 9*$/10
        bullet.momy = 9*$/10
    end
end


local function BulletMoveBlocked(bullet, enemy, wall)
    -- Bullet flies into the wall
    if(wall ~= nil) then 
        S_StartSound(wall.frontsector, sfx_miss)    
    -- Bullet hits the entity
    else
        S_StartSound(enemy, sfx_hit)
    end
end

addHook("PlayerThink", function(player) 
    if(Valid(player.mo, "jazz") and player.tf_mode == TF_MODES.BOT_MODE) then
        BotPlayerThink(player) 
    end
end)

addHook("MobjThinker", function(mo)
    BulletMobjThinker(mo)
end, MT_BULL)

addHook("MobjMoveBlocked", function(mo, tmmo, line)
    BulletMoveBlocked(mo, tmmo, line)
end, MT_BULL)
